"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AbstractAmqpPublisher = void 0;
const node_core_1 = require("@lokalise/node-core");
const core_1 = require("@message-queue-toolkit/core");
const AbstractAmqpService_1 = require("./AbstractAmqpService");
class AbstractAmqpPublisher extends AbstractAmqpService_1.AbstractAmqpService {
    messageSchemaContainer;
    exchange;
    initPromise;
    constructor(dependencies, options) {
        super(dependencies, options);
        this.messageSchemaContainer = this.resolvePublisherMessageSchemaContainer(options);
        this.exchange = options.exchange;
    }
    publish(message, options) {
        const resolveSchemaResult = this.resolveSchema(message);
        if (resolveSchemaResult.error) {
            throw resolveSchemaResult.error;
        }
        resolveSchemaResult.result.parse(message);
        // If it's not initted yet, do the lazy init
        if (!this.isInitted) {
            // avoid multiple concurrent inits
            if (!this.initPromise) {
                this.initPromise = this.init();
            }
            /**
             * it is intentional that this promise is not awaited, that's how we keep the method invocation synchronous
             * RabbitMQ publish by itself doesn't guarantee that your message is delivered successfully, so this kind of fire-and-forget is not strongly different from how amqp-lib behaves in the first place.
             */
            this.initPromise
                .then(() => {
                this.publish(message, options);
            })
                .catch((err) => {
                this.handleError(err);
            });
            return;
        }
        if (this.logMessages) {
            // @ts-ignore
            const resolvedLogMessage = this.resolveMessageLog(message, message[this.messageTypeField]);
            this.logMessage(resolvedLogMessage);
        }
        // biome-ignore lint/style/noParameterAssign: it's ok
        message = this.updateInternalProperties(message);
        try {
            this.publishInternal((0, core_1.objectToBuffer)(message), options);
        }
        catch (err) {
            // Unfortunately, reliable retry mechanism can't be implemented with try-catch block,
            // as not all failures end up here. If connection is closed programmatically, it works fine,
            // but if server closes connection unexpectedly (e. g. RabbitMQ is shut down), then we don't land here
            // @ts-ignore
            if (err.message === 'Channel closed') {
                this.logger.error(`AMQP channel closed`);
                void this.reconnect();
            }
            else {
                throw new node_core_1.InternalError({
                    message: `Error while publishing to AMQP ${err.message}`,
                    errorCode: 'AMQP_PUBLISH_ERROR',
                    details: (0, node_core_1.copyWithoutUndefined)({
                        publisher: this.constructor.name,
                        // @ts-ignore
                        queueName: this.queueName,
                        exchange: this.exchange,
                        // @ts-ignore
                        messageType: message[this.messageTypeField] ?? 'unknown',
                    }),
                    cause: err,
                });
            }
        }
    }
    resolveSchema(message) {
        return this.messageSchemaContainer.resolveSchema(message);
    }
    /* c8 ignore start */
    resolveMessage() {
        throw new Error('Not implemented for publisher');
    }
    /* c8 ignore start */
    processPrehandlers() {
        throw new Error('Not implemented for publisher');
    }
    preHandlerBarrier() {
        throw new Error('Not implemented for publisher');
    }
    resolveNextFunction() {
        throw new Error('Not implemented for publisher');
    }
    async close() {
        this.initPromise = undefined;
        await super.close();
    }
    processMessage() {
        throw new Error('Not implemented for publisher');
    }
}
exports.AbstractAmqpPublisher = AbstractAmqpPublisher;
//# sourceMappingURL=AbstractAmqpPublisher.js.map